package com.winhc.bigdata.spark.implicits

import java.time.LocalDate
import java.time.format.DateTimeFormatter

import org.apache.commons.lang3.StringUtils
import org.apache.spark.broadcast.Broadcast
import org.apache.spark.sql.DataFrame
import org.apache.spark.sql.functions.col
import org.elasticsearch.spark._

/**
 * @Author: XuJiakai
 * @Date: 2020/7/11 16:25
 * @Description:
 */
object CompanyIndexSave2EsHelper {
  val companyIndexFields = Seq(
    "cid"
    , "name"
    , "history_names"
    , "current_cid"
    , "company_type" //公司类型

    , "credit_code" //统一信用代码
    , "reg_status" //公司状态
    , "area_code" // 行政区划码
    , "reg_location" //注册地址
    , "estiblish_time" // 成立日期(注册时间)
    , "lat" //公司纬度
    , "lng" //公司经度
    , "category_code" // 行业分类
    , "reg_capital" // 注册资本,展示类型
    , "reg_capital_amount" // 注册资本,数值类型
    , "phones" //电话
    , "emails" //邮箱
  )

  implicit class DataFrameEnhancer(df: DataFrame) {
    def companyIndexSave2Es(category_code: Broadcast[Map[String, Seq[String]]], area_code: Broadcast[Map[String, Seq[String]]]): Unit = {
      df.select(companyIndexFields.map(column => col(column).cast("string")): _*)
        .rdd.map(r => {
        val map = companyIndexFields.map(f => {
          (f, r.getAs[String](f))
        }).toMap
        getEsDoc(map, category_code, area_code)
      })
        .saveToEsWithMeta("winhc-company-v6/company")
    }

   /* def test2Es(spark:SparkSession,category_code: Broadcast[Map[String, Seq[String]]], area_code: Broadcast[Map[String, Seq[String]]]): Unit = {
      val rdd = df.select(companyIndexFields.map(column => col(column).cast("string")): _*)
        .rdd.map(r => {
        val map = companyIndexFields.map(f => {
          (f, r.getAs[String](f))
        }).toMap
        getEsDoc(map, category_code, area_code)
      }).map(r=>Row(r._1,r._2.estiblish_time))
      spark.createDataFrame(rdd,StructType(Array(StructField("cid",StringType),StructField("time",StringType))))
       .write.mode("overwrite")
       .insertInto("winhc_eci_dev.xjk_test_save_2_es_0721")
    }*/
  }

  case class Geo(lat: String, lon: String)

  case class CompanyName(show: String, value: String)

  case class CompanyDoc(
                         cname: CompanyName
                         , current_id: String
                         , history_name: Seq[CompanyName]
                         , company_type: String
                         , credit_code: String
                         , reg_status: String
                         //                         , geo: Geo
                         , province_code: String
                         , province_name: String
                         , city_code: String
                         , city_name: String
                         , county_code: String
                         , county_name: String
                         , reg_location: String
                         , estiblish_time: String
                         , category_code: String
                         , category_first: String
                         , category_second: String
                         , category_third: String
                         , reg_capital: String
                         , reg_capital_amount: String
                         , phones: Seq[String]
                         , emails: Seq[String]
                       )

  val pattern = "[^\\u4e00-\\u9fa50-9a-zA-Z]".r
  //  val time_pattern = ""

  def getEsDoc(map: Map[String, String], category_map: Broadcast[Map[String, Seq[String]]], area_map: Broadcast[Map[String, Seq[String]]]): (String, CompanyDoc) = {
    val lat = map("lat")
    val lng = map("lng")
    var geo: String = null
    if (StringUtils.isNotBlank(lat) && StringUtils.isNotBlank(lng)) {
      geo = lat + "," + lng
    }

    val c = get_area_code(map("area_code"), area_map)
    val province_code = c._1
    val province_name = c._2
    val city_code = c._3
    val city_name = c._4
    val county_code = c._5
    val county_name = c._6

    val category = get_category_code(map("category_code"), category_map)
    val category_code = category._1
    val category_first = category._2
    val category_second = category._3
    val category_third = category._4

    val et = map("estiblish_time")
    var time = if (StringUtils.isNotBlank(et)) {
      if (et.contains(" ")) {
        et.split(" ")(0)
      } else {
        et
      }
    } else null

    if(!validateDf(time)){
      time = null
    }

    val doc = CompanyDoc(
      cname = getCompanyName(map("name"))
      , current_id = map("current_cid")
      , history_name = getHistoryName(map("name"), map("history_names"))
      , company_type = map("company_type")
      , credit_code = map("credit_code")
      , reg_status = map("reg_status")
      //      , geo = Geo(lat = lng, lon = lat)
      , province_code = province_code
      , province_name = province_name
      , city_code = city_code
      , city_name = city_name
      , county_code = county_code
      , county_name = county_name
      , reg_location = map("reg_location")
      , estiblish_time = time
      , category_code = category_code
      , category_first = category_first
      , category_second = category_second
      , category_third = category_third
      , reg_capital = map("reg_capital")
      , reg_capital_amount = map("reg_capital_amount")
      , phones = getSplit(map("phones"))
      , emails = getSplit(map("emails"))
    )
    (map("cid"), doc)
  }


  private def getCompanyName(name: String): CompanyName = {
    if (StringUtils.isEmpty(name)) null
    else {
      val value = pattern replaceAllIn(name, "")
      CompanyName(name, value)
    }
  }

  private def getHistoryName(cname: String, names: String): Seq[CompanyName] = {
    if (StringUtils.isEmpty(names)) {
      null
    } else {
      val res = getSplit(names)
        .filter(!cname.equals(_))
        .filter(StringUtils.isNoneEmpty(_))
        .map(getCompanyName)
      if (res.isEmpty) {
        null
      } else {
        res
      }
    }
  }

  private def get_area_code(code: String, area_code: Broadcast[Map[String, Seq[String]]]): (String, String, String, String, String, String) = {
    if (StringUtils.isNotBlank(code) && code.trim.length == 6) {
      val c = code.trim
      (c.substring(0, 2), get_seq_by_index(area_code, c, 0), c.substring(2, 4), get_seq_by_index(area_code, c, 1), c.substring(4, 6), get_seq_by_index(area_code, c, 2))
    } else {
      (null, null, null, null, null, null)
    }
  }

  private def get_category_code(code: String, category_code: Broadcast[Map[String, Seq[String]]]): (String, String, String, String) = {
    if (StringUtils.isNotBlank(code)) {
      val c = code.trim
      (c, get_seq_by_index(category_code, c, 0), get_seq_by_index(category_code, c, 1), get_seq_by_index(category_code, c, 2))
    } else {
      (null, null, null, null)
    }
  }

  def get_seq_by_index(area_code: Broadcast[Map[String, Seq[String]]], code: String, index: Int): String = {
    val c = area_code.value.getOrElse(code, null)
    if (c == null) null else if (c(index) != null) c(index).trim else null
  }

  private def getSplit(str: String): Seq[String] = {
    if (StringUtils.isNotBlank(str)) {
      str.split("\t;\t").filter(StringUtils.isNotBlank).toSet.toList
    } else {
      Seq.empty[String]
    }
  }

  private val df = DateTimeFormatter.ofPattern("yyyy-MM-dd")

  private def validateDf(str: String): Boolean = try {
    if (StringUtils.isNotBlank(str)) {
      LocalDate.parse(str, df)
      true
    } else {
      false
    }
  } catch {
    case ex: java.time.format.DateTimeParseException => {
      false
    }
  }

  def main(args: Array[String]): Unit = {
    println(validateDf("2010-03-03"))
  }
}
