package com.winhc.bigdata.spark.implicits

import com.winhc.bigdata.spark.config.HBaseConfig
import org.apache.hadoop.hbase.client.Put
import org.apache.hadoop.hbase.io.ImmutableBytesWritable
import org.apache.hadoop.hbase.util.Bytes
import org.apache.spark.sql.DataFrame

/**
 * @Author: XuJiakai
 * @Date: 2020/7/29 13:51
 * @Description:
 */
object DataFrame2HBaseHelper {

  implicit class DataFrameEnhancer(df: DataFrame) {

    def save2HBase(tableName: String, rowkeyFieldName: String, fields: Seq[String], f_bytes: Array[Byte] = Bytes.toBytes("F")): Unit = {
      import org.apache.spark.sql.functions.col
      val jobConf = HBaseConfig.HBaseOutputJobConf(tableName)

      val stringDf = df.select(df.columns.map(column => col(column).cast("string")): _*)
      stringDf.rdd.map(row => {
        val id = row.getAs[String](rowkeyFieldName)
        val put = new Put(Bytes.toBytes(id))
        for (f <- fields) {
          val v = row.getAs[String](f.toLowerCase)
          if (v != null) {
            put.addColumn(f_bytes, Bytes.toBytes(f.toUpperCase()), Bytes.toBytes(v))
          }
        }
        (new ImmutableBytesWritable, put)
      }).filter(_ != null)
        .saveAsHadoopDataset(jobConf)
    }
  }
}
