package com.winhc.bigdata.spark.jobs

import com.winhc.bigdata.spark.config.HBaseConfig
import com.winhc.bigdata.spark.utils.{CompanyNameMappingUtil, SparkUtils}
import org.apache.hadoop.hbase.TableName
import org.apache.hadoop.hbase.spark.HBaseContext
import org.apache.hadoop.hbase.spark.HBaseRDDFunctions._
import org.apache.hadoop.hbase.util.Bytes
import org.apache.spark.internal.Logging
import org.apache.spark.sql.Row
import org.apache.spark.sql.types.{StringType, StructField, StructType}

/**
 * @Author: XuJiakai
 * @Date: 2020/5/26 10:41
 * @Description: 公司名称映射关系计算  只获取最新的公司id
 */
object CompanyNameMappingPro_stage_01 extends Logging {
  val f_bytes: Array[Byte] = Bytes.toBytes("f")
  val cname_bytes: Array[Byte] = Bytes.toBytes("name")
  val current_cid_bytes: Array[Byte] = Bytes.toBytes("current_cid")

  def main(args: Array[String]): Unit = {
    val inputTable = "xjk_company_name_mapping_tmp"
    val hbaseKVTable = "company_name_kv"

    val spark = SparkUtils.InitEnv("CompanyNameMappingPro_stage_01")
    import spark._
    /**
     * 写hbase,供查询
     */
    val jobConf = HBaseConfig.HBaseOutputJobConf(hbaseKVTable)

    val df_old = sql(
      s"""
         |SELECT  cid
         |        ,cname
         |        ,current_cid
         |        ,new_cid
         |        ,new_cname
         |        ,flag
         |FROM    $inputTable
         |WHERE new_cid is null
         |""".stripMargin).repartition(50)

    val hbaseContext = new HBaseContext(spark.sparkContext, jobConf)
    /**
     * 查hbase,找到最新的公司id
     */
    val tmp_rdd = df_old.rdd.hbaseMapPartitions(hbaseContext, (f, con) => {
      val table = con.getTable(TableName.valueOf(hbaseKVTable))
      val rdd_par = f.map(record => {
        val cid = record.getString(0)
        val name = record.getString(1)
        val current_cid = record.getAs[String](2)
        try {
          val (res_cid, res_name) = CompanyNameMappingUtil.getCurrentIdAndName(table, current_cid)
          Row(cid, name, current_cid, res_cid, res_name, "0")
        } catch {
          case e: Exception => {
            logWarning(record.toString())
            logError(e.getMessage, e)
          }
            Row(cid, name, current_cid, null, null, "-1")
        }
      })
      rdd_par
    }).filter(_ != null)

    val schema = StructType(Array(
      StructField("cid", StringType),
      StructField("cname", StringType),
      StructField("current_cid", StringType),
      StructField("new_cid", StringType),
      StructField("new_cname", StringType),
      StructField("flag", StringType)
    ))

    val tmp_df = spark.createDataFrame(tmp_rdd, schema)

    val df = sql(
      """
        |SELECT  cid
        |        ,cname
        |        ,current_cid
        |        ,new_cid
        |        ,new_cname
        |        ,flag
        |FROM    xjk_company_name_mapping_tmp
        |WHERE new_cid is not null
        |""".stripMargin)
    tmp_df.union(df).write.mode("overwrite").insertInto(inputTable)

    logInfo("CompanyNameMapping success")
    spark.stop()
  }
}
