package com.winhc.bigdata.spark.jobs.dynamic.tables

import com.winhc.bigdata.spark.jobs.dynamic.CompanyDynamicHandle
import com.winhc.bigdata.spark.implicits.BaseHelper._
/**
 * @Author: XuJiakai
 * @Date: 2020/8/6 17:07
 * @Description:
 */
case class company() extends CompanyDynamicHandle {

  private val change_type_map = Map(
    "name" -> "公司名称"
    , "legal_entity_id" -> "法人"
    , "company_org_type" -> "公司类型"
    , "reg_location" -> "注册地址"
    , "to_time" -> "营业期限"
    , "business_scope" -> "经营范围"
    , "reg_institute" -> "登记机关"
    , "reg_status" -> "企业状态"
    , "reg_capital" -> "注册资本"
    , "actual_capital_amount" -> "实收资本"
  )


  /**
   * 来源表的变更类型，insert or update
   *
   * @return
   */
  override def org_type(): Seq[String] = Seq("update")

  private def makeRiskLevelHandle(field: String, old_map: Map[String, String], new_map: Map[String, String]) = field match {
    case "actual_capital_amount" | "reg_capital" => {

      val old_val = old_map.getOrElse(field, "")
      val new_val = new_map.getOrElse(field, "")
      if (old_val.isRegCapital && new_val.isRegCapital) {
        old_val.toRegCapital().compareTo(new_val.toRegCapital()) match {
          case 1 => {
            "3"
          }
          case -1 => {
            "1"
          }
          case 0 => {
            null
          }
        }
      } else "2"
    }
    case _ => "2"

  }

  /**
   *
   * @param rowkey
   * @param cid
   * @param change_fields
   * @param old_map
   * @param new_map
   * @return cid
   *         cname
   *         info_type
   *         rta_desc
   *         change_content
   *         change_time
   *         biz_id
   *         sub_info_type
   *         info_risk_level
   *         winhc_suggest
   */
  override def handle(rowkey: String, bizDate: String, cid: String, change_fields: Seq[String], old_map: Map[String, String], new_map: Map[String, String], cname: String, suggestion: String): Seq[(String, String, String, String, String, String, String, String, String, String)] = {
    change_fields.filter(e => change_type_map.contains(e)).map(e => {
      val v = makeRiskLevelHandle(e, old_map, new_map)
      if (v == null) {
        return null
      }
      (
        rowkey
        , new_map("name")
        , super.get_info_type()
        , s"${change_type_map(e)}发生变更"
        , s"""{"变更后内容": ${new_map(e).getOrNull()},"变更事项": ${change_type_map(e).getOrNull()},"变更日期": "$bizDate","变更前内容": ${old_map(e).getOrNull()}}"""
        , bizDate
        , rowkey
        , super.get_sub_info_type()
        , v
        , null
      )
    })
  }

  /**
   * 信息描述
   *
   * @param old_map
   * @param new_map
   * @return
   */
  override protected def get_rta_desc(old_map: Map[String, String], new_map: Map[String, String]): String = null

}
