package com.winhc.bigdata.spark.jobs.dynamic.tables

import com.winhc.bigdata.spark.jobs.dynamic.CompanyDynamicHandle
import com.winhc.bigdata.spark.implicits.MapHelper._

/**
 * @Author: XuJiakai
 * @Date: 2020/7/31 14:59
 * @Description: 股权出质
 */
case class company_equity_info() extends CompanyDynamicHandle {

  /**
   *
   * @param rowkey
   * @param cid
   * @param change_fields
   * @param old_map
   * @param new_map
   * @return cid
   *         cname
   *         info_type
   *         rta_desc
   *         change_content
   *         change_time
   *         biz_id
   *         sub_info_type
   *         info_risk_level
   *         winhc_suggest
   */
  override def handle(rowkey: String, bizDate: String, cid: String, change_fields: Seq[String], old_map: Map[String, String], new_map: Map[String, String], cname: String, suggestion: String): Seq[(String, String, String, String, String, String, String, String, String, String)] = {
    if ("1".equals(new_map("deleted"))) {
      return Seq.empty
    }
    var list: Seq[(String, String, String)] = Seq((new_map("cid"), cname, "2")) //标的企业
    if ("2".equals(new_map.getOrElse("pledgor_type", "0"))) {
      list = list :+ (new_map("pledgor_id"), new_map("pledgor"), "3") //出质人
    }
    if ("2".equals(new_map.getOrElse("pledgee_type", "0"))) {
      list = list :+ (new_map("pledgee_id"), new_map("pledgee"), "1") //质权人
    }

    list.map(t => {
      (t._1
        , t._2
        , get_info_type()
        , get_rta_desc(old_map, new_map)
        , get_change_content(old_map, new_map, cname)
        , get_change_time(bizDate, new_map)
        , get_biz_id(rowkey)
        , get_sub_info_type()
        , t._3
        , "被监控企业流动资金紧张，可能存在经营困难的情况。建议立即与被监控企业书面对账，适当催促其履行债务并持续监控。"
      )
    }).seq
  }

  /**
   * 信息描述
   *
   * @param old_map
   * @param new_map
   * @return
   */
  override protected def get_rta_desc(old_map: Map[String, String], new_map: Map[String, String]): String = "新增1条股权出质信息"

  /**
   * 变更内容
   *
   * @param old_map
   * @param new_map
   * @return
   */
  override protected def get_change_content(old_map: Map[String, String], new_map: Map[String, String], cname: String = null): String = {
    val json = new_map.toJson(Seq("equity_amount->出质股权数额", "state->出质状态", "reg_date->股权出质设立登记日期", "reg_number->质权登记编号"))
    json.substring(0, json.length - 1) +
      s""","出质股权标的企业": {"企业名称": "$cname","企业KeyNo": "${new_map("cid")}"},"出质人信息": {"证件号": "${new_map("certif_number_l")}","出质人": "${new_map("pledgor")}"},"质权人信息": {"质权人": "${new_map("pledgee")}","证件号": "${new_map("certif_number_r")}"}}"""
  }

  /**
   * 风险等级
   *
   * @param old_map
   * @param new_map
   * @return
   */
  override protected def get_info_risk_level(old_map: Map[String, String], new_map: Map[String, String]): String = null
}
