package com.winhc.bigdata.spark.jobs.dynamic.tables

import com.winhc.bigdata.spark.jobs.dynamic.CompanyDynamicHandle

/**
 * @Author yyn
 * @Date 2020/8/4
 * @Description
 */
//土地转让
case class company_land_transfer() extends CompanyDynamicHandle {
  /**
   * 信息描述
   *
   * @param old_map
   * @param new_map
   * @return
   */
  override def get_rta_desc(old_map: Map[String, String], new_map: Map[String, String]): String = new_map("num")

  /**
   * 变更内容
   *
   * @param old_map
   * @param new_map
   * @return
   */
  override def get_change_content(old_map: Map[String, String], new_map: Map[String, String], cname: String = null): String = {
    s"""转让价格:${new_map("merchandise_price")}万元\n
       |成交时间：${new_map("merchandise_time")}\n""".stripMargin
  }

  /**
   * 变更时间
   *
   * @param new_map
   * @return
   */
//  override def get_change_time(new_map: Map[String, String]): String = new_map("biz_date")

  /**
   * 风险等级
   *
   * @param old_map
   * @param new_map
   * @return
   */
  override def get_info_risk_level(old_map: Map[String, String], new_map: Map[String, String]): String = {
    //获取抵押类型：now：只有现有土地使用权人；pre：只有原土地使用权人；bothsame：原土地使用权人与现土地使用权人为同一人或企业（有这样的数据）；bothone：两者数据均有的原土地使用人；bothtwo：两者数据均有的现有土地使用人
    val mortType = new_map("type")
    if (mortType != null && !mortType.isEmpty) {
      //现有土地使用权人
      if(mortType.equals("now") || mortType.equals("bothtwo")) {
        "提示信息"
      }
      else{//原有土地使用权人
        "警示信息"
      }
    }
    else{
      null//脏数据
    }
  }
  /**
   *
   * @param rowkey
   * @param cid
   * @param change_fields
   * @param old_map
   * @param new_map
   * @return cid
   *         cname
   *         info_type
   *         rta_desc
   *         change_content
   *         change_time
   *         biz_id
   *         sub_info_type
   *         info_risk_level
   *         winhc_suggest
   */
  override def handle(rowkey: String, bizDate: String, cid: String, change_fields: Seq[String], old_map: Map[String, String], new_map: Map[String, String], cname: String, suggestion: String): Seq[(String, String, String, String, String, String, String, String, String, String)] = {
//获取抵押类型：mortgagee：只有抵押权人；mortgagor：只有抵押人；bothsame：同为抵押人和抵押权人（有这样的数据）；bothone：两者数据均有的抵押人；bothtwo：两者数据均有的抵押权人
    val mortType = new_map("type")
    if (mortType != null && !mortType.isEmpty) {
      //现有土地使用权人
      if(mortType.equals("now") || mortType.equals("bothtwo")) {
        super.handle(rowkey, bizDate, cid, change_fields, old_map, new_map, cname, "该企业有接受其他企业土地转让行为")
      }
      else{//原有土地使用权人
        super.handle(rowkey, bizDate, cid, change_fields, old_map, new_map, cname, "该企业有土地转让行为")
      }
    }
    else{
      null//脏数据
    }
  }
}