package com.winhc.bigdata.spark.jobs.dynamic.tables

import com.winhc.bigdata.spark.jobs.dynamic.CompanyDynamicHandle
import com.winhc.bigdata.spark.implicits.MapHelper._
import org.apache.commons.lang3.StringUtils

/**
 * @Author: π
 * @Date: 2020/8/11 17:39
 * @Description: 税收违法
 */
case class company_tax_contravention() extends CompanyDynamicHandle {


  /**
   * 来源表的变更类型，insert or update
   *
   * @return
   */
  override def org_type(): Seq[String] = Seq("insert")

  /**
   * 信息描述
   *
   * @param old_map
   * @param new_map
   * @return
   */
  override protected def get_rta_desc(old_map: Map[String, String], new_map: Map[String, String]): String = {
    s"""
       |案件性质：${new_map("case_type")}\n
       |发布日期：${new_map("publish_time")}
       |""".stripMargin
  }

  /**
   * 变更内容
   *
   * @param old_map
   * @param new_map
   * @return
   */
  override protected def get_change_content(old_map: Map[String, String], new_map: Map[String, String], cname: String): String = {
      new_map.toJson(Seq(
        "taxpayer_name->纳税人名称"
        , "taxpayer_number->纳税人识别号"
        , "taxpayer_code->组织机构代码"
        , "address->注册地址"
        , "publish_time->发布日期"
        , "case_type->案件性质"
        , "department->所属税务机关"
        , "case_info->主要违法事实"
        , "legal_person_info->法定代表人或负责人"
//        , s"${splitInfo(new_map("legal_person_info"))._2}->性别"
//        , s"${splitInfo(new_map("legal_person_info"))._3}->证件名称"
//        , s"${splitInfo(new_map("legal_person_info"))._4}->证件号码"
        , "responsible_person_info->负有责任的财务负责人"
//        , s"${splitInfo(new_map("legal_person_info"))._2}->性别"
//        , s"${splitInfo(new_map("legal_person_info"))._3}->证件名称"
//        , s"${splitInfo(new_map("legal_person_info"))._4}->证件号码"
        , "responsible_department_info->负有直接责任的中介机构"
      ))
  }

  def splitInfo(s: String) = {
    if (StringUtils.isNotBlank(s) && s.replaceAll(":", ",").split(",").size == 4) {
      val Array(name, sex, card, number) = s.replaceAll(":", ",").split(",")
      (name, sex, card, number)
    } else {
      ("", "", "", "")
    }
  }

  override protected def get_info_risk_level(old_map: Map[String, String], new_map: Map[String, String]): String = {
    "3"
  }
}
