package com.winhc.bigdata.spark.test

import java.util.concurrent.CountDownLatch

import com.winhc.bigdata.spark.config.EsConfig
import com.winhc.bigdata.spark.jobs.dynamic.CompanyDynamic.{CompanyDynamicUtil}
import com.winhc.bigdata.spark.utils.{SparkUtils, Watching}
import org.apache.spark.sql.SparkSession

import scala.collection.mutable

/**
 * @Description: ${todo}
 * @author π
 * @date 2020/8/1918:08
 */
object TestCompanyDynamic {

  private case class Args(project: String = "winhc_eci_dev"
                          , tableName: String
                          , bName: Int = 1)

  private val startArgs = Seq(
    Args(tableName = "company_abnormal_info", bName = 0)
    , Args(tableName = "company_equity_info")
    , Args(tableName = "company_staff", bName = 0)
    , Args(tableName = "company", bName = 0)
    , Args(tableName = "bankruptcy_open_case", bName = 1)
    , Args(tableName = "company_illegal_info", bName = 0)
    , Args(tableName = "company_land_publicity", bName = 1)
    , Args(tableName = "company_employment", bName = 1)
    , Args(tableName = "company_land_announcement", bName = 1)
    , Args(tableName = "company_bid_list", bName = 1)
    , Args(tableName = "company_land_transfer", bName = 1)
    , Args(tableName = "company_env_punishment", bName = 1)
    , Args(tableName = "company_punishment_info", bName = 1)
    , Args(tableName = "company_punishment_info_creditchina", bName = 1)
    , Args(tableName = "bankruptcy_open_case", bName = 1)
    , Args(tableName = "company_public_announcement2_list", bName = 1)
    , Args(tableName = "company_mortgage_info", bName = 1)
    , Args(tableName = "company_stock_announcement", bName = 1)
    , Args(tableName = "company_finance", bName = 1)
    , Args(tableName = "company_dishonest_info", bName = 1)
    , Args(tableName = "company_send_announcement_list", bName = 1)
    , Args(tableName = "company_annual_report_out_guarantee", bName = 1)
    , Args(tableName = "company_zxr_restrict", bName = 1)

    , Args(tableName = "company_zxr_final_case", bName = 1) //终本案件
    , Args(tableName = "company_license_creditchina", bName = 1) //行政许可-信用中国
    , Args(tableName = "company_license_entpub", bName = 1) //行政许可-企业公示
    , Args(tableName = "company_license", bName = 1) //行政许可
    , Args(tableName = "company_check_info", bName = 1) //抽查检查
    , Args(tableName = "company_court_announcement_list", bName = 1) //法院公告
    , Args(tableName = "company_court_open_announcement_list", bName = 1) //开庭公告
    , Args(tableName = "company_court_register_list", bName = 1) //立案信息
    , Args(tableName = "company_double_random_check_info", bName = 1) //双随机抽查
    , Args(tableName = "company_judicial_sale_combine_list", bName = 1) //司法拍卖
    , Args(tableName = "company_tax_contravention", bName = 1) //税收违法
    , Args(tableName = "wenshu_detail_combine", bName = 1) //裁判文书
  )
  val seq = List(
    "company_mortgage_info"
    , "company_check_info"
    , "company_court_announcement_list"
    , "company_court_open_announcement_list"
    , "company_court_register_list"
    , "company_double_random_check_info"
    , "company_judicial_sale_combine_list"
    , "company_tax_contravention"
    , "company_zxr_final_case"
    , "company_license_creditchina"
    , "company_license_entpub"
    , "company_license"
    , "wenshu_detail_combine"
  )

  def main(args: Array[String]): Unit = {

    val project = "winhc_eci_dev"

    val config = EsConfig.getEsConfigMap ++ mutable.Map(
      "spark.hadoop.odps.project.name" -> project,
      "spark.hadoop.odps.spark.local.partition.amt" -> "1000"
    )
    val spark = SparkUtils.InitEnv("CompanyDynamic", config)
    TestCompanyDynamic(spark, project, "20200817").calc()
    spark.stop()
  }


  case class TestCompanyDynamic(s: SparkSession,
                                project: String, //表名（不加前后辍）
                                ds: String //表名（不加前后辍）
                               ) extends Watching {

    override protected val spark: SparkSession = s

    def calc2(): Unit = {
      startArgs.filter(s => seq.contains(s.tableName)).foreach(e => {
        println("______________________________" + e.tableName + "___________________________________")
        CompanyDynamicUtil(spark, project, ds).calc(e.tableName, e.bName)
      })
    }

    def calc(): Unit = {
      val latch = new CountDownLatch(seq.length)
      startArgs.filter(s => seq.contains(s.tableName)).foreach(e => {
        asyncWatch(e.tableName, () => {
          println("______________________________" + e.tableName + "___________________________________")
          CompanyDynamicUtil(spark, project, ds).calc(e.tableName, e.bName)
          latch.countDown()
        })
      })
      latch.await()
    }

  }

}
