package com.winhc.bigdata.spark.udf

import com.winhc.bigdata.calc.DimScore
import org.apache.commons.lang3.StringUtils
import org.apache.spark.broadcast.Broadcast
import org.apache.spark.sql.SparkSession
import com.winhc.bigdata.spark.utils.BaseUtil._

trait CompanyMapping {

  def prepareFunctions(spark: SparkSession): Unit ={
    import spark._
    //清理特殊字符
    spark.udf.register("cleanup", (col: String) => {
      cleanup(col)
    })

    spark.udf.register("replace_char", (col: String) => {
      replaceChar(col)
    })

    spark.udf.register("case_no", (col: String) => {
      caseNo(col)
    })
  }

  def prepare(spark: SparkSession): Unit = {
    import spark._
    //注册资本
    spark.udf.register("reg_capital_score", (capital: String) => {
      DimScore.registEvaluate(capital)
    })

    //实缴资本金额转换
    spark.udf.register("actual_capital_score", (actual_capital: String, reg_capital: String) => {
      DimScore.actualEvaluate(actual_capital, reg_capital);
    })

    //所属行业
    val code2Name: Broadcast[Map[String, String]] = spark.sparkContext.broadcast(sql(
      """
        |select category_code,category_str_big
        |from const_company_category_code
      """.stripMargin).collect().map(r => {
      (r.getString(0), r.getString(1))
    }).toMap)

    spark.udf.register("industry_score", (code: String) => {
      DimScore.industryEvaluate(code2Name.value.getOrElse(code, null))
    })

    //企业类型
    spark.udf.register("company_type_score", (company_org_type: String) => {
      DimScore.companyTypeEvaluate(company_org_type);
    })

    //企业状态
    spark.udf.register("company_status_score", (reg_status: String) => {
      DimScore.companyStatusEvaluate(reg_status);
    })

    //成立日期
    spark.udf.register("start_date_score", (estiblish_time: String) => {
      DimScore.startDateEvaluate(estiblish_time);
    })

    //企业期限
    spark.udf.register("end_date_score", (from_time: String, to_time: String) => {
      DimScore.endDateEvaluate(from_time, to_time);
    })

    //注册地址
    spark.udf.register("reg_location_score", (reg_location: String) => {
      DimScore.addressEvaluate(reg_location);
    })

  }

}
